const std = @import("std");

const Digraph = @import("./Digraph.zig");

pub const Node        = @import("./Node.zig");
pub const NodeList    = std.ArrayList(*Node);
pub const NodeDigraph = Digraph.Container(*const Node, Digraph.lessThanFuncFor(*const Node));

///
alloc: std.mem.Allocator,

///
nodes: NodeList,

///
digraph: NodeDigraph,

///
root: *Node,

///
pub fn init(alloc: std.mem.Allocator) !@This() {
    var nodes = NodeList.init(alloc);
    errdefer nodes.deinit();

    var digraph = try NodeDigraph.init(alloc, &.{});
    errdefer digraph.deinit();

    var root = try alloc.create(Node);
    errdefer alloc.destroy(root);

    root.* = try Node.init(alloc, 0, "helloworld");
    errdefer root.deinit(alloc);

    try nodes.append(root);

    var node1 = try alloc.create(Node);
    errdefer alloc.destroy(node1);
    node1.* = try Node.init(alloc, 1, "node1");
    errdefer node1.deinit(alloc);
    try nodes.append(node1);
    try digraph.connect(root, node1);

    var node2 = try alloc.create(Node);
    errdefer alloc.destroy(node2);
    node2.* = try Node.init(alloc, 2, "node1");
    errdefer node2.deinit(alloc);
    try nodes.append(node2);
    try digraph.connect(root, node2);

    return .{
        .alloc   = alloc,
        .nodes   = nodes,
        .digraph = digraph,
        .root    = root,
    };
}
///
pub fn deinit(self: *@This()) void {
    for (self.nodes.items) |node| {
        node.deinit(self.alloc);
        self.alloc.destroy(node);
    }
    self.digraph.deinit();
    self.nodes.deinit();
}

///
pub fn serialize(self: *const @This(), writer: anytype) !void {
    _ = self;
    _ = writer;
}
///
pub fn deserialize(reader: anytype) !@This() {
    _ = reader;
}
