const std    = @import("std");
const dvui   = @import("dvui");
const hncore = @import("hncore");

///
pub fn manipulator(root: *const hncore.Mindmap.Node, digraph: *const hncore.Mindmap.NodeDigraph) !void {
    var overlay = try dvui.overlay(@src(), .{
        .expand = .both,
    });
    defer overlay.deinit();

    _ = try nodeTreeInManipulator(.{ .x = 100, .y = 100, }, root, digraph);
}

///
pub fn nodeTreeInManipulator(base: dvui.Point, node: *const hncore.Mindmap.Node, digraph: *const hncore.Mindmap.NodeDigraph) !dvui.Rect {
    const size = try nodeInManipulator(base, node);
    const childrenRect = nodeChildrenInManipulator(
        .{ .x = base.x + size.x, .y = base.y, }, node, digraph);
    return .{
        .x = base.x,
        .y = base.y,
        .w = childrenRect.w + size.w,
        .h = @max(childrenRect.h, size.h),
    };
}

///
pub fn nodeInManipulator(base: dvui.Point, node: *const hncore.Mindmap.Node) !dvui.Rect {
    var box = try dvui.box(@src(), .vertical, .{
        .id_extra = node.id,
        .border   = .all(1),
        .rect     = .{ .x = base.x, .y = base.y, .w = 100, .h = 20, },
    });
    defer box.deinit();
    return box.wd.borderRect();
}

///
pub fn nodeChildrenInManipulator(base: dvui.Point, node: *const hncore.Mindmap.Node, digraph: *const hncore.Mindmap.NodeDigraph) dvui.Rect {
    const parentMargin  = 16;
    const siblingMargin = 8;

    var childDepth : f32 = 0;
    var childOffset: f32 = 0;
    for (digraph.getChildrenOf(node)) |conn| {
        const childRect = nodeTreeInManipulator(
            .{ .x = base.x + parentMargin, .y = base.y + childOffset, }, conn.to, digraph) catch unreachable;
        childOffset += childRect.h + siblingMargin;
        childDepth   = @max(childRect.w, childDepth);
    }
    return .{ .x = base.x, .y = base.y, .w = childDepth, .h = childOffset, };
}
